from .email_util import send_interface
from .person import Person
from .solver import loop, draw
from .text_util import save


def run(person_list, invalid_links=None, **kwargs):
    """Solve and output. Raises SolvingError if it cannot solve."""
    if invalid_links is None:
        invalid_links = dict()
    if not isinstance(person_list, list):
        raise ValueError('person_list must be a list')
    if not isinstance(invalid_links, dict):
        raise ValueError('invalid_links must be a dict')
    if not all(isinstance(p, Person) for p in person_list):
        raise ValueError('Every item in person_list must be a Person')

    message = kwargs.get('message', '')
    subject = kwargs.get('subject', 'Secret Santa')
    username = kwargs.get('username', None)
    password = kwargs.get('password', None)
    out_mode = kwargs.get('out_mode', 'email').lower()
    solve_mode = kwargs.get('solve_mode', 'loop').lower()

    # solve
    if solve_mode == 'loop':
        solved = loop(person_list, invalid_links)
    elif solve_mode == 'draw':
        solved = draw(person_list, invalid_links)
    else:
        raise ValueError('Unknown solve mode "{}"'.format(solve_mode))

    # output
    if out_mode == 'email':
        failed = send_interface(solved, message, subject, username, password)  # send emails (errors are returned)
        if failed:
            print('{} failures: {}'.format(len(failed), failed))
    elif out_mode == 'text':
        save(solved)
    else:
        raise ValueError('Unknown output mode "{}"'.format(out_mode))



def save(solved_dict):
    for gifter, recipient in solved_dict.items():
        with open('Assignment for {}.txt'.format(gifter.name), 'w') as f:
            f.write('Your recipient is {}.\n'.format(recipient.name))



from distutils.core import setup
from os import path

MAJOR = 0
MINOR = 2
PATCH = 1

HERE = path.abspath(path.dirname(__file__))
try:
    with open(path.join(HERE, 'README.rst'), encoding='utf-8') as fp:
        README = fp.read()
except FileNotFoundError:
    README = ''

setup(
    name='santa',
    packages=['santa'],
    version='{}.{}.{}'.format(MAJOR, MINOR, PATCH),
    description='Assign a secret santa over email',
    long_description=README,
    author='jarhill0',
    author_email='jarhill0@gmail.com',
    url='https://github.com/jarhill0/santa/',
    download_url='https://github.com/jarhill0/santa/archive/{}.{}.{}.tar.gz'.format(MAJOR, MINOR, PATCH),
    keywords=['secret', 'santa', 'email'],
    classifiers=[],
    install_requires=[
        'yagmail >=0.10.209',
    ]
)
